import monobank
from datetime import datetime
from monobank.utils import to_timestamp
from monobank.signature import SignKey
from monobank.transport import api_request


class ClientBase(object):
    
    def _get_headers(self, url):
        raise NotImplementedError('Please implement _get_headers')

    def make_request(self, path):
        headers = self._get_headers(path)
        return api_request('GET', path, headers=headers)

    def get_currency(self):
        return self.make_request('/bank/currency')

    def get_client_info(self):
        return self.make_request('/personal/client-info')
    
    def get_statements(self, account, date_from, date_to):
        assert date_from <= date_to
        t_from, t_to = to_timestamp(date_from), to_timestamp(date_to)
        url = f'/personal/statement/{account}/{t_from}/{t_to}'
        return self.make_request(url)


class Client(ClientBase):
    "Personal API"

    def __init__(self, token):
        self.token = token

    def _get_headers(self, url):
        return {
            'X-Token': self.token,
        }


class CorporateClient(ClientBase):
    "Corporate API"

    def __init__(self, request_id, private_key):
        self.request_id = request_id
        self.key = SignKey(private_key)
        
    def _get_headers(self, url):
        headers = {
            'X-Key-Id': self.key.key_id(),
            'X-Time': str(to_timestamp(datetime.now())),
            'X-Request-Id': self.request_id,
        }
        data = headers['X-Time'] + headers['X-Request-Id'] + url
        headers['X-Sign'] = self.key.sign(data)
        return headers
    
    def check(self):
        "Checks if user approved access request"
        try:
            self.make_request('/personal/auth/request')
            return True
        except monobank.Error as e:
            if e.response.status_code == 401:
                return False
            raise


def access_request(permissions, private_key, callback_url=None):
    "Creates an access request for corporate api user"
    key = SignKey(private_key)
    headers = {
        'X-Key-Id': key.key_id(),
        'X-Time': str(to_timestamp(datetime.now())),
        'X-Permissions': permissions,
    }
    if callback_url:
        headers['X-Callback'] = callback_url
    path = '/personal/auth/request'
    sign_str = headers['X-Time'] + headers['X-Permissions'] + path
    headers['X-Sign'] = key.sign(sign_str)
    return api_request('POST', path, headers=headers)




import os
import ecdsa
import base64
import hashlib
import binascii


class SignKey(object):
    def __init__(self, priv_key):
        self.priv_key = priv_key
        self.sk = self._load()
    
    def key_id(self):
        "Returns monobank X-Key-Id"
        publicKey = self.sk.get_verifying_key()
        publicKeyB64 = base64.b64encode(publicKey.to_der())

        uncompressedPublicKey = bytearray([0x04]) + (bytearray(publicKey.to_string()))
        digests = hashlib.sha1()
        digests.update(uncompressedPublicKey)
        return binascii.hexlify(digests.digest())
    
    def sign(self, str_to_sign):
        "Signs string str_to_sign with private key, and hash sha256"
        sign = self.sk.sign(str_to_sign.encode(), hashfunc=hashlib.sha256)
        return base64.b64encode(sign)

    def _load(self):
        if 'PRIVATE KEY-----' in self.priv_key:
            raw = self.priv_key
        elif os.path.exists(self.priv_key):
            with open(self.priv_key) as f:
                raw = f.read()
        else:
            raise Exception('Cannot load private key')
        return ecdsa.SigningKey.from_pem(raw)



import requests
import monobank

ENDPOINT = 'https://api.monobank.ua'
UAGENT = 'python-monobank (https://github.com/vitalik/python-monobank, contact: ppr.vitaly@gmail.com)'


def api_request(method, path, **kwargs):
    "Handles all HTTP requests for monobank endponts"
    headers = kwargs.pop('headers')
    headers['User-Agent'] = UAGENT
    url = ENDPOINT + path
    # print(method, url, headers)
    response = requests.request(method, url, headers=headers, **kwargs)
    
    if response.status_code == 200:
        return response.json()
    
    if response.status_code == 429:
        raise monobank.TooManyRequests("Too many requests", response)
    
    data = response.json()
    message = data.get('errorDescription', str(data))
    raise monobank.Error(message, response)



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['monobank']

package_data = \
{'': ['*']}

install_requires = \
['ecdsa>=0.13.2,<0.14.0', 'requests>=2.22,<3.0']

setup_kwargs = {
    'name': 'monobank',
    'version': '0.4.1',
    'description': 'Monobank.ua API implementation',
    'long_description': '# python-monobank\n\n![python-monobank](https://raw.githubusercontent.com/vitalik/python-monobank/master/docs/logo.png)\n\nPython client for Monobank API (https://api.monobank.ua/docs/)\n\n## Installation\n\n```\npip install monobank\n```\n\n\n# Usage\n\n## Personal api\n\n1) Request your token at https://api.monobank.ua/\n\n2) Use that token to initialize client:\n\n```python\n  import monobank\n  token = \'xxxxxxxxxxxxxxx\'\n\n  mono = monobank.Client(token)\n  user_info = mono.get_client_info()\n  print(user_info)\n```\n\n### Methods\n\nGet currencies\n\n```python\n>>> mono.get_currency()\n[\n {\'currencyCodeA\': 840,\n  \'currencyCodeB\': 980,\n  \'date\': 1561686005,\n  \'rateBuy\': 25.911,\n  \'rateSell\': 26.2357},\n {\'currencyCodeA\': 978,\n  \'currencyCodeB\': 980,\n  \'date\': 1561686005,\n  \'rateBuy\': 29.111,\n  \'rateSell\': 29.7513},\n  ...\n```\n\nGet client info\n\n```python\n>>> mono.get_client_info()\n{\n  \'name\': \'Dmitriy Dubilet\'\n  \'accounts\': [\n    {\n      \'id\': \'accidxxxxx\'\n      \'balance\': 100000000,\n      \'cashbackType\': \'UAH\',\n      \'creditLimit\': 100000000,\n      \'currencyCode\': 980,\n      }\n  ],\n}\n\n```\n\n\nGet statements\n```python\n>>> mono.get_statements(\'accidxxxxx\', date(2019,1,1), date(2019,1,30))\n[\n  {\n    \'id\': \'iZDPhf8v32Qass\',\n    \'amount\': -127603,\n    \'balance\': 99872397,\n    \'cashbackAmount\': 2552,\n    \'commissionRate\': 0,\n    \'currencyCode\': 978,\n    \'description\': \'Smartass club\',\n    \'hold\': True,\n    \'mcc\': 5411,\n    \'operationAmount\': 4289,\n    \'time\': 1561658263\n  },\n  ...\n]\n```\n\nYou can as well pass datetime objects\n```python\n>>> mono.get_statements(\'accidxxxxx\', datetime(2019,1,1,11,15), datetime(2019,1,2,11,15))\n```\n\n\n\n\n## Corporatre API\n\nDocumentation is here - https://api.monobank.ua/docs/corporate.html\n\nCorporate API have the same methods as Public API, but it does not have rate limitation, and it is a recomended way if you are handling data for commercial use (or just storing lot of personal data).\n\n### Getting access\n\n#### 1) Generate private key\n\n```\nopenssl ecparam -genkey -name secp256k1 -rand /dev/urandom -out priv.key\n```\n\nThis will output file **priv.key** \n\n**Warning**: do not share it with anyone, do not store it in public git repositories\n\n#### 2) Generate public key\n\n```\nopenssl ec -in priv.key  -pubout -out pub.key\n```\n\nThis will output file **pub.key** \n\n#### 3) Request API access \nSend an email to api@monobank.ua - describe your project, and attach **pub.key** (!!! NOT priv.key !!!)\n\n\n### Requesting permission from monobank user\n\nOnce your app got approved by Monobank team you can start using corporate API:\n\n\n#### 1) Create monobank user access request\n\n```python\nprivate_key = \'/path/to/your/priv.key\'\nrequest = monobank.access_request(\'p\', private_key)\n```\nIf all fine you should recive the following:\n```python\nprint(request)\n{\'tokenRequestId\': \'abcdefg_Wg\', \'acceptUrl\': \'https://mbnk.app/auth/abcdefg_Wg\'}\n```\n\nYou should save tokenRequestId to database, and then give user the link acceptUrl\n\nNote: To be notified about user acceptance you can use web callback:\n\n```python\nmonobank.access_request(\'p\', private_key, callback_url=\'https://yourserver.com/callback/\')\n```\n\n#### 2) Check if user accepted\n\nYou can check if user accepted access request like this:\n\n\n```python\nrequest_token = \'abcdefg_Wg\'  # the token from access_request result\nprivate_key = \'/path/to/your/priv.key\'\n\nmono = monobank.CorporateClient(request_token, private_key)\n\n\nmono.check()  # returns True if user accepted, False if not\n\n```\n\n\n#### 3) Use methods\n\nOnce user accepts your access-request, you can start using all the methods same ways as Public API\n\n```python\nmono.get_statements(....)\n```\n\n## Handling Errors\n\nIf you use Personal API you may encounter "Too Many Requests" error. To properly catch it and retry - use *monobank.TooManyRequests* exception\n\n```python\ntry:\n    mono.get_statements(....)\nexcept monobank.TooManyRequest:\n    time.sleep(1)\n    # try again:\n    mono.get_statements(....)\n```\n\nYou can use ratelimiter library (like https://pypi.org/project/ratelimiter/ ) to download all transactions\n',
    'author': 'Vitaliy Kucheryaviy',
    'author_email': 'ppr.vitaly@gmail.com',
    'url': 'https://github.com/vitalik/python-monobank',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)




class Error(Exception):
    def __init__(self, message, response):
        super().__init__(message)
        self.response = response
    
    def __str__(self):
        return f'{self.response.status_code}: {self.args[0]}'


class TooManyRequests(Error):
    pass
